package com.gyee.alarm.service.auto.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gyee.alarm.init.CacheContext;
import com.gyee.alarm.mapper.auto.ProEconShutdownEvent2Mapper;
import com.gyee.alarm.model.auto.ProEconShutdownEvent2;
import com.gyee.alarm.model.vo.ShutdownEventVo;
import com.gyee.alarm.service.auto.IProEconShutdownEvent2Service;
import com.gyee.alarm.util.DateUtils;
import com.gyee.common.model.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 停机事件2 服务实现类
 * </p>
 *
 * @author shilin
 * @since 2022-10-21
 */
@Service
public class ProEconShutdownEvent2ServiceImpl extends ServiceImpl<ProEconShutdownEvent2Mapper, ProEconShutdownEvent2> implements IProEconShutdownEvent2Service {

    @Resource
    private ProEconShutdownEvent2Mapper proEconShutdownEvent2Mapper;
    public Map<String,Object> queryShutdownEvent2(String wpId, String wtId, String description, String begin, String end, Integer pageNum, Integer pageSize, String type, String modelId) {

        Map<String,Object> map=new HashMap();

        if (StringUtils.notEmp(begin) && StringUtils.notEmp(end)) {
            try {
                Date beginDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(begin);
                Date endDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(end);
                List<ProEconShutdownEvent2> ls=proEconShutdownEvent2Mapper.queryShutdownEvent( wpId, modelId, beginDate,  endDate,type,wtId);

                map.put("total",ls.size());
                List<ProEconShutdownEvent2> vos = paginate(ls, pageNum, pageSize);


                for(ProEconShutdownEvent2 vo:vos)
                {
                    if(com.gyee.alarm.util.StringUtils.notEmp(vo.getWindturbineId()) && CacheContext.wtmap.containsKey(vo.getWindturbineId()))
                    {
                        vo.setCode(CacheContext.wtmap.get(vo.getWindturbineId()).getNemCode());

                        vo.setWindturbineName(CacheContext.wtmap.get(vo.getWindturbineId()).getName());


                    }


                }
                map.put("result",vos);

            } catch (ParseException e) {
                e.printStackTrace();
            }
        }else
        {
            map.put("total",0);
            map.put("result", new ArrayList<ProEconShutdownEvent2>());
        }


        return map;
    }

    /**
     * 对集合进行分页
     *
     * @param <T>        集合中元素的类型
     * @param list       要分页的集合
     * @param pageNumber 页码，从1开始
     * @param pageSize   每页显示的元素数量
     * @return 返回指定页的数据
     */
    public static <T> List<T> paginate(List<T> list, int pageNumber, int pageSize) {
        if (list == null || list.isEmpty() || pageNumber <= 0 || pageSize <= 0) {
            return new ArrayList<>();
        }

        // 计算开始和结束的索引
        int fromIndex = (pageNumber - 1) * pageSize;
        if (fromIndex >= list.size()) {
            return new ArrayList<>();
        }

        int toIndex = Math.min(fromIndex + pageSize, list.size());
        return new ArrayList<>(list.subList(fromIndex, toIndex));
    }

    public  List<ShutdownEventVo> queryShutdownEvent2ByType(String wpId, String modelId, Date begin, Date end)
    {
        return proEconShutdownEvent2Mapper.queryShutdownEvent2ByType(wpId, modelId, begin,  end);
    }


    public List<ShutdownEventVo> queryShutdownEvent2ByDay(String wpId,String modelId,Date begin, Date end)
    {
        List<ShutdownEventVo> result=new ArrayList<>();
        List<String> dayls= DateUtils.getCurrentMonthDateStrings();
        List<ShutdownEventVo> vos= proEconShutdownEvent2Mapper.queryShutdownEvent2ByDay(wpId, modelId, begin,  end);

        Map<String,ShutdownEventVo> map=new HashMap<>();
        if(!vos.isEmpty()) {

            for(ShutdownEventVo vo:vos)
            {
                map.put(vo.getStoptime(),vo);
            }
        }
        if(!dayls.isEmpty())
        {
            for(String str:dayls)
            {
                ShutdownEventVo vo=new ShutdownEventVo();
                vo.setStoptime(str);
                if(map.containsKey(str))
                {
                    vo.setHours(map.get(str).getHours());
                    vo.setTimes(map.get(str).getTimes());
                }else
                {
                    vo.setHours(0.0);
                    vo.setTimes(0l);
                }
                result.add(vo);
            }
        }

        return result;
    }


    public List<ShutdownEventVo> queryShutdownEvent2ByMonth(String wpId,String modelId,Date begin, Date end)
    {

        List<ShutdownEventVo> result=new ArrayList<>();
        List<String> monthls= DateUtils.getYearMonths();
        List<ShutdownEventVo> vos=proEconShutdownEvent2Mapper.queryShutdownEvent2ByMonth(wpId, modelId, begin,  end);

        Map<String,ShutdownEventVo> map=new HashMap<>();
        if(!vos.isEmpty()) {

            for(ShutdownEventVo vo:vos)
            {
                map.put(vo.getStoptime(),vo);
            }
        }
        if(!monthls.isEmpty())
        {
            for(String str:monthls)
            {
                ShutdownEventVo vo=new ShutdownEventVo();
                vo.setStoptime(str);
                if(map.containsKey(str))
                {
                    vo.setHours(map.get(str).getHours());
                    vo.setTimes(map.get(str).getTimes());
                }else
                {
                    vo.setHours(0.0);
                    vo.setTimes(0l);
                }
                result.add(vo);
            }
        }

        return result;
    }


    public  List<ShutdownEventVo> queryShutdownEvent2ByJX(String wpId,String modelId,Date begin, Date end)
    {
        return proEconShutdownEvent2Mapper.queryShutdownEvent2ByJX(wpId, modelId, begin,  end);
    }

    public List<ShutdownEventVo> queryShutdownEvent2ByDQ(String wpId,String modelId,Date begin, Date end)
    {
        return proEconShutdownEvent2Mapper.queryShutdownEvent2ByDQ(wpId, modelId, begin,  end);
    }

    public List<ShutdownEventVo> queryShutdownEvent2ByOther(String wpId,String modelId,Date begin, Date end)
    {
        return proEconShutdownEvent2Mapper.queryShutdownEvent2ByOther(wpId, modelId, begin,  end);
    }
}
